import { Provider } from "@infra-blocks/types";
/**
 * Typed enum of possible GitHub actions events.
 *
 * This is meant to be extended as we create more GitHub actions
 */
export declare enum Event {
    Push = 0,
    PullRequest = 1
}
/**
 * Parses the event name and returns the corresponding {@link Event}
 *
 * @param eventName - The name of the event, as provided by the GitHub context.
 */
export declare function parseEvent(eventName: string): Event;
/**
 * Parses and verifies that the event is part of the supported event.
 *
 * This is useful in the context of actions where you don't expect most events to trigger the
 * action.
 *
 * If the event is supported, then it is returned. Otherwise, an error is thrown.
 *
 * @param eventName - The event name, as provided by the GitHub context.
 * @param supported - The list of supported events by the action.
 *
 * @return The parsed event, if it's supported.
 */
export declare function checkSupportedEvent(eventName: string, supported: ReadonlyArray<Event>): Event;
/**
 * Validator objects used to extract GitHub actions values.
 */
export interface InputValidator<T> {
    parse(input: string | undefined): T;
}
export declare function stringInput(options?: Record<string, never>): InputValidator<string>;
export declare function stringInput(options: {
    default: string | Provider<string>;
}): InputValidator<string>;
export declare function stringInput(options: {
    default: undefined | Provider<string | undefined>;
}): InputValidator<string | undefined>;
export declare function stringInput<T extends string>(options: {
    choices: T[];
}): InputValidator<T>;
export declare function stringInput<T extends string>(options: {
    default: T;
    choices: T[];
}): InputValidator<T>;
export declare function stringInput<T extends string>(options: {
    default: undefined;
    choices: T[];
}): InputValidator<T | undefined>;
export declare function arrayInput(options?: {
    separator?: string | RegExp;
    trim?: boolean;
}): InputValidator<ReadonlyArray<string>>;
export declare function arrayInput(options: {
    default: ReadonlyArray<string> | Provider<ReadonlyArray<string>>;
    separator?: string | RegExp;
    trim?: boolean;
}): InputValidator<ReadonlyArray<string>>;
export declare function arrayInput(options: {
    default: undefined | Provider<ReadonlyArray<string> | undefined>;
    separator?: string | RegExp;
    trim?: boolean;
}): InputValidator<ReadonlyArray<string> | undefined>;
export declare function booleanInput(): InputValidator<boolean>;
export declare function booleanInput(options: {
    default: boolean | Provider<boolean>;
}): InputValidator<boolean>;
export declare function booleanInput(options: {
    default: undefined | Provider<boolean | undefined>;
}): InputValidator<boolean | undefined>;
/**
 * Returns a type safe snapshot of the provided GitHub actions inputs.
 *
 * @param inputValidators - An object where the keys are the name of the inputs
 * and the values are their matching validators.
 */
export declare function getInputs<T>(inputValidators: {
    [K in keyof T]: InputValidator<T[K]>;
}): Readonly<T>;
/**
 * GitHub action outputs are simple strings.
 *
 * Whatever isn't a string is JSON stringified. To be more explicit about it,
 * this framework requires calling code to stringify themselves.
 *
 */
export type Outputs = Record<string, string>;
/**
 * Parses the Github Actions outputs from the provided file as written by the core.setOutput
 * utility.
 *
 * The core.setOutput utility stores the outputs as heredocs with randomly generated delimiters.
 * This function parses those out and returns the remaining key value pairs as an object.
 *
 * If a key is provided more than once, the last value takes precedence.
 *
 * @param filePath - The outputs file path. Defaults to GITHUB_OUTPUT. Throws if neither the
 *  parameter and the environment variable are provided.
 *
 * @return The outputs as a record of string.
 */
export declare function parseOutputs(filePath?: string): Promise<Outputs>;
export type ActionHandler<I> = (params: {
    inputs: Readonly<I>;
}) => Promise<Outputs>;
export declare function runActionHandler<I>(handler: ActionHandler<I>, options: {
    inputValidators: {
        [K in keyof I]: InputValidator<I[K]>;
    };
}): void;
export declare function runActionHandler(handler: () => Promise<Outputs>): void;
export declare function runActionHandler(handler: () => Promise<Outputs>, options: Record<string, never>): void;
