import VError from "verror";
import * as core from "@actions/core";
import { readFile } from "node:fs/promises";
import { isFunction } from "@infra-blocks/types";
/**
 * Typed enum of possible GitHub actions events.
 *
 * This is meant to be extended as we create more GitHub actions
 */
export var Event;
(function (Event) {
    Event[Event["Push"] = 0] = "Push";
    Event[Event["PullRequest"] = 1] = "PullRequest";
})(Event = Event || (Event = {}));
/**
 * Parses the event name and returns the corresponding {@link Event}
 *
 * @param eventName - The name of the event, as provided by the GitHub context.
 */
export function parseEvent(eventName) {
    switch (eventName) {
        case "push":
            return Event.Push;
        case "pull_request":
            return Event.PullRequest;
        default:
            throw new Error(`unsupported event: ${eventName}`);
    }
}
/**
 * Parses and verifies that the event is part of the supported event.
 *
 * This is useful in the context of actions where you don't expect most events to trigger the
 * action.
 *
 * If the event is supported, then it is returned. Otherwise, an error is thrown.
 *
 * @param eventName - The event name, as provided by the GitHub context.
 * @param supported - The list of supported events by the action.
 *
 * @return The parsed event, if it's supported.
 */
export function checkSupportedEvent(eventName, supported) {
    const event = parseEvent(eventName);
    if (supported.includes(event)) {
        return event;
    }
    throw new Error(`unsupported event: ${eventName}`);
}
function parseInput(input, transform, options) {
    const required = options == null ||
        !Object.prototype.hasOwnProperty.call(options, "default");
    // GitHub actions actually provide the value as an empty string when it is missing at the time
    // of this writing.
    if (input == null || input === "") {
        if (required) {
            throw Error("input is missing value and no default was provided");
        }
        if (options == null) {
            throw new Error("unexpected null options");
        }
        if (isFunction(options.default)) {
            try {
                return options.default();
            }
            catch (err) {
                throw new VError({ cause: err }, `default value provider threw`);
            }
        }
        return options.default;
    }
    return transform(input);
}
/**
 * Returns a validator for string inputs.
 *
 * @param options.default - If defined, the input becomes optional and when
 *  not found, the default value is returned. The default can also be a {@link Provider}. In which case, it is
 *  only called when the input isn't found.
 */
export function stringInput(options) {
    const { choices } = options || {};
    return {
        parse(input) {
            return parseInput(input, (input) => {
                if (choices == null || choices.length === 0) {
                    return input;
                }
                if (!choices.includes(input)) {
                    throw new Error(`invalid value: ${input} for string input with choices: ${JSON.stringify(choices)}`);
                }
                return input;
            }, options);
        },
    };
}
/**
 * Returns a validator for arrays parsed out of strings.
 *
 * The default separator is the comma, but users can specify other
 * separators.
 *
 * @param options.default - If defined, the input becomes optional and when
 *  not found, the default value is returned. The default can also be a {@link Provider}. In which case, it is
 *  only called when the input isn't found.
 * @param options.separator - The token separator. Defaults to ",".
 * @param options.trim - Whether to trim the array tokens. False by default.
 */
export function arrayInput(options) {
    const { separator = ",", trim = false } = options || {};
    return {
        parse(input) {
            return parseInput(input, (input) => {
                const tokens = input.split(separator);
                if (!trim) {
                    return tokens;
                }
                return tokens.map((token) => token.trim());
            }, options);
        },
    };
}
/**
 * Returns a validator for boolean inputs.
 *
 * @param options.default - If defined, the input becomes optional and when
 *  not found, the default value is returned. The default can also be a {@link Provider}. In which case, it is
 *  only called when the input isn't found.
 */
export function booleanInput(options) {
    return {
        parse(input) {
            function transform(input) {
                if (input === "true") {
                    return true;
                }
                if (input === "false") {
                    return false;
                }
                throw new Error(`invalid boolean input: ${input}`);
            }
            return parseInput(input, transform, options);
        },
    };
}
/**
 * Returns a type safe snapshot of the provided GitHub actions inputs.
 *
 * @param inputValidators - An object where the keys are the name of the inputs
 * and the values are their matching validators.
 */
export function getInputs(inputValidators) {
    const result = {};
    for (const [name, value] of Object.entries(inputValidators)) {
        const input = getInput(name);
        try {
            result[name] = value.parse(input);
        }
        catch (err) {
            throw new VError({ name: "GetInputsError", cause: err }, `error parsing input ${name}`);
        }
    }
    return result;
}
/**
 * Returns the value of the input, as found in the environment.
 *
 * This function was copied over from @actions/core, just because we wanted to avoid having
 * a dependency for this single functionality.
 *
 * @param name - The input name.
 */
function getInput(name) {
    return process.env[`INPUT_${name.replace(/ /g, "_").toUpperCase()}`];
}
/**
 * Parses the Github Actions outputs from the provided file as written by the core.setOutput
 * utility.
 *
 * The core.setOutput utility stores the outputs as heredocs with randomly generated delimiters.
 * This function parses those out and returns the remaining key value pairs as an object.
 *
 * If a key is provided more than once, the last value takes precedence.
 *
 * @param filePath - The outputs file path. Defaults to GITHUB_OUTPUT. Throws if neither the
 *  parameter and the environment variable are provided.
 *
 * @return The outputs as a record of string.
 */
export async function parseOutputs(filePath) {
    const outputsPath = filePath || process.env.GITHUB_OUTPUT;
    if (outputsPath == null) {
        throw new Error(`no output file path provided as argument nor through the GITHUB_OUTPUT environment variable`);
    }
    const result = {};
    const fileContents = await readFile(outputsPath, { encoding: "utf-8" });
    const outputRegex = /(?<key>.+?)<<(?<delimiter>.*?)\n(?<value>.*)\n\k<delimiter>\n/g;
    let captures = outputRegex.exec(fileContents);
    while (captures != null && captures.groups != null) {
        result[captures.groups.key] = captures.groups.value;
        captures = outputRegex.exec(fileContents);
    }
    return result;
}
/**
 * This function does away with the common boilerplate code related to running a GitHub Actions
 * handler.
 *
 * The inputs are formally declared with the input validators argument. They are extracted out of the
 * environment and passed on to the handler.
 *
 * The handler may or may not return {@link Outputs}. In the latter case, an empty object is expected.
 *
 * The outputs are forwarded to core.setOutput.
 *
 * The function also wraps the whole process with convenient debug statements that are turned on
 * by setting ACTIONS_STEP_DEBUG to true.
 *
 * Any runtime errors occurring during this function's execution results in a call to core.setFailed.
 *
 * @param handler - The GitHub Actions handler.
 * @param options.inputValidators - The set of validators to extract the inputs from the environment.
 *
 * @see https://docs.github.com/en/actions/monitoring-and-troubleshooting-workflows/enabling-debug-logging
 */
export function runActionHandler(handler, options) {
    const { inputValidators } = options || {};
    try {
        if (core.isDebug()) {
            core.debug(`received env: ${JSON.stringify(process.env, null, 2)}`);
            core.debug(`received context: ${JSON.stringify(context, null, 2)}`);
        }
        let promise;
        if (inputValidators == null) {
            core.debug("no inputs specified");
            promise = handler();
        }
        else {
            const inputs = getInputs(inputValidators);
            if (core.isDebug()) {
                core.debug(`parsed out inputs: ${JSON.stringify(inputs)}`);
            }
            promise = handler({ inputs });
        }
        promise
            .then((outputs) => {
            for (const [key, value] of Object.entries(outputs)) {
                if (core.isDebug()) {
                    core.debug(`setting output ${key}=${value}`);
                }
                core.setOutput(key, value);
            }
        })
            .catch((err) => core.setFailed(VError.fullStack(err)));
    }
    catch (err) {
        core.setFailed(VError.fullStack(err));
    }
}
//# sourceMappingURL=github-action.js.map